//** Modified-Light version of Texture for TP**// 

#include "Texture.h"


Texture::Texture()
{
  picture = NULL;
  N_x=-1;
  N_y=-1;
}

Texture::~Texture()
{
  destroy();
}

int Texture::destroy()
{
  if(picture!=NULL)
    {
      delete[] picture;
      picture = NULL;
    }

  N_x=-1;
  N_y=-1;

  return 0;
}


int Texture::load_picture(const char* file_name)
{
  int ok=0;


  int file_type = get_file_type(file_name);
  ok+=destroy();

  switch(file_type)
    {
    case PPM_FORMAT:
      load_ppm_picture(file_name);
      break;
    case NO_FORMAT:
      cout<<"WARNING NO known format for "<<file_name<<endl;
      return -1;
    }


  return ok;
}

int Texture::load_ppm_picture(const char* _file_name)
{
  
#define SIZE_BUFFER 2048
  char buffer[SIZE_BUFFER]={'\0'};

  FILE *fid=NULL;  

  char file_name[100];
  strcpy(file_name,_file_name);
  //strcat(file_name,".ppm");

  fid = fopen(file_name,"r");
  if(fid==NULL)
    {cout<<"ERROR read file "<<file_name<<"\n";exit (-1);}


  //lokk for P3 magic number
  buffer[0]='#';
  while(buffer[0]=='#')
    fgets(buffer,sizeof(buffer),fid);
  if(strcmp(buffer,"P3\n")!=0)
    {printf("Error reading file %s, can only read P3 type, here it is a %s type\n",file_name,buffer);exit(-1);}
  
  buffer[0]='#';
  while(buffer[0]=='#')
    fgets(buffer,sizeof(buffer),fid);

  //read size
  N_x=0;N_y=0;
  if(sscanf(buffer,"%d %d\n",&N_x,&N_y)!=2)
    {printf("Error in file %s, cannot read size\n",file_name);exit(-1);}
  fgets(buffer,sizeof(buffer),fid);//get max value


  if(picture!=NULL)
    {
      free (picture);
      picture=NULL;
    }
  picture = (unsigned char*)malloc(4*N_x*N_y*sizeof(unsigned char));

  // now read values
  int k_x,k_y=0,k_dim=0;

  for(k_x=0;k_x<N_x;k_x++)
    for(k_y=0;k_y<N_y;k_y++)
      {
	for(k_dim=0;k_dim<3;k_dim++)
	  {
	    if(fscanf(fid,"%s",buffer)!=1){printf("Error reading value in file %s\n",file_name);exit(-1);}
	    picture[4*(k_y+N_y*k_x)+k_dim]=(unsigned char)(atoi(buffer));
	  }
	picture[4*(k_x+N_x*k_y)+3]=255;
      }

  if(fclose(fid)!=0)
    {printf("Error closing file %s\n",file_name);exit(-1);}

  //only store RGBA values
  depth=4;

  return 0;
}

int Texture::export_ppm(const char* _file_name)
{
  int ok=0;
  char file_name[100];

  
  FILE *fid_file=NULL;  
  
  //export in ppm

  if(picture==NULL)
    exit(-1);

  strcpy(file_name,_file_name);
  strcat(file_name,".ppm");

  fid_file = fopen(file_name,"w");
  if(fid_file==NULL)
    {
      cout<<"ERROR CANNOT CREATE "<<file_name<<"\n";
      return (-1);
    }

  fprintf(fid_file,"P3\n");//magic number
  fprintf(fid_file,"# %s.ppm\n",file_name);//name of the file in comment
  fprintf(fid_file,"%d %d\n",N_x,N_y);//size of the file
  fprintf(fid_file,"255\n");//max value

  int k;
  for(k=0;k<N_x*N_y;k++)
    fprintf(fid_file,"%d \n%d \n%d \n",picture[4*k],picture[4*k+1],picture[4*k+2]);

  ok+=fclose(fid_file);

  return ok;
}

int Texture::get_Nx()
{
  return N_x;
}
int Texture::get_Ny()
{
  return N_y;
}
int Texture::get_nearest_texel(float s_1,float s_2,float *texel)
{
  int ok=0;
  int us_1=0,us_2=0;
  int k_texture=0;
  int k_dim=0;

  if(picture==NULL)
    exit(-1);

  //nearest
  us_1 = int(fabs(s_1*float(N_x)+0.5f))%N_x;
  us_2 = int(fabs(s_2*float(N_y)+0.5f))%N_y;

  k_texture = 4*(us_1+us_2*N_x);


  for(k_dim=0;k_dim<3;k_dim++)
    {
      texel[k_dim] = float((picture[k_texture+k_dim])/256.0);
    }

  return ok;
}


int Texture::get_linear_texel(float s_1,float s_2,float *texel)
{
  int ok=0;

  float us_1=0.0,us_2=0.0;
  int k_1=0,k_2=0;

  s_1=s_1-float(int(s_1));
  s_2=s_2-float(int(s_2));

  us_1 = s_1*float(N_x);
  us_2 = s_2*float(N_y);

  k_1 = (int)us_1;
  k_2 = (int)us_2;

  //linear interpolant
  float t_x = (us_1-float(k_1));
  float t_y = (us_2-float(k_2));



  if(picture==NULL)
    {
      cout<<"ERROR NO PICTURE in texture interpolation\n";
      exit(-1);
    }

  int k_10=k_1,k_11=k_1+1;
  int k_20=k_2,k_21=k_2+1;

  //protection on boundaries
  if(k_11>=N_x)
    k_11=k_10;
  if(k_21>=N_y)
    k_21=k_20;

  int X00 = k_10 + k_20*N_x;
  int X01 = k_11 + k_20*N_x;
  int X10 = k_10 + k_21*N_x;
  int X11 = k_11 + k_21*N_x;


  int k_dim=0;

  for(k_dim=0;k_dim<3;k_dim++)
    {
      texel[k_dim] = 
	    t_x *     t_y * (float)picture[4*X11+k_dim]+
	(1-t_x) *     t_y * (float)picture[4*X10+k_dim]+
	    t_x * (1-t_y) * (float)picture[4*X01+k_dim]+
	(1-t_x) * (1-t_y) * (float)picture[4*X00+k_dim];
    }



  return ok;
}


unsigned char* Texture::get_data()
{
  return picture;
}


int Texture::get_file_type(const char* filename)
{
  int file_type = NO_FORMAT;
  char *buffer = NULL;

  //get last dot and return extension
  buffer = strrchr(filename,'.');

  if(strcmp(buffer+1,"ppm")==0)
    file_type = PPM_FORMAT;
  
  return file_type;
}

int Texture::get_depth()
{
  return depth;
}
