 
/********************************************************/
/* Chargement fichier off                               */
/********************************************************/

/* inclusion des fichiers d'en-tete Glut */


#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <iostream>
#include <GL/glut.h>
#include <vector>
#include "int_vector.h"
#include "Texture.h"

#define PI 3.14159


std::vector <double> vertex;
std::vector <double> normal;
std::vector <double> color;
std::vector <double> texture;
std::vector <int> connectivity;
GLuint texture_number=0;

std::vector <std::vector<int> > face_ring;

int drawing_number=-1;

char presse,presse_d=0;
int anglex,angley,x,y,xold,yold;
double zoom=-5.0;

/* Prototype des fonctions */
void affichage();
void clavier(unsigned char touche,int x,int y);
void reshape(int x,int y);
void idle();
void mouse(int bouton,int etat,int x,int y);
void mousemotion(int x,int y);

void build_normal();
void scale();
void create_drawing_list();
void load_torus();
void load_texture_file(const char* filename);

int main(int argc,char **argv)
{
  /* initialisation de glut et creation
     de la fenetre */
  glutInit(&argc,argv);
  glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
  glutInitWindowPosition(0,0);
  glutInitWindowSize(550,550);
  glutCreateWindow("TP-VM-RICM3");

  /* Initialisation d'OpenGL */
  glClearColor(1.0,1.0,1.0,0.0);
  glColor3f(1.0,1.0,1.0);
  glPointSize(2.0);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_COLOR_MATERIAL);

  /* enregistrement des fonctions de rappel */
  glutDisplayFunc(affichage);
  glutKeyboardFunc(clavier);
  glutReshapeFunc(reshape);
  glutMouseFunc(mouse);
  glutMotionFunc(mousemotion);


  //light
  float light_ambient[4]={0.4,0.4,0.4,1.0};
  float light_diffuse[4]={0.4,0.4,0.4,1.0};
  float light_specular[4]={0.3,0.3,0.3,1.0};
  float light_position [4] = {2.0,100.0,100.0,1.0};

  float material_ambient [4] = {1.0,1.0,1.0,1.0};
  float material_diffuse [4] = {1.0,1.0,1.0,1.0};
  float material_specular [4] = {1.0,1.0,1.0,1.0};
  float material_exponent [1] = {60.0};


  glShadeModel(GL_SMOOTH);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
  glLightfv(GL_LIGHT0, GL_POSITION, light_position);
  glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
  glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);

  glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, material_ambient);
  glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, material_diffuse);  
  glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, material_specular);  
  glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, material_exponent);
  glEnable(GL_DEPTH_TEST);





  /* load torus */
  load_torus(); // a modifier ...

  /* load a ppm file of texture*/
  load_texture_file("image_test.ppm");

  /* Calcule les normales */
  build_normal();

  /* center and scale size */
  scale();

  


  /* drawing list */
  create_drawing_list();

  /* Entree dans la boucle principale glut */
  glutMainLoop();
  return 0;
}

void load_torus()
{

  // A modifier .. //

  vertex.push_back(0);  vertex.push_back(0);  vertex.push_back(0);
  vertex.push_back(1);  vertex.push_back(0);  vertex.push_back(0);
  vertex.push_back(0);  vertex.push_back(1);  vertex.push_back(0);
  vertex.push_back(1);  vertex.push_back(1);  vertex.push_back(0);

  connectivity.push_back(0); connectivity.push_back(1); connectivity.push_back(3); connectivity.push_back(2);

  color.push_back(1.0); color.push_back(0.0); color.push_back(0.0);
  color.push_back(0.0); color.push_back(1.0); color.push_back(0.0);
  color.push_back(0.0); color.push_back(0.0); color.push_back(1.0);
  color.push_back(1.0); color.push_back(1.0); color.push_back(0.0);

  texture.push_back(0.0); texture.push_back(0.0);
  texture.push_back(1.0); texture.push_back(0.0);
  texture.push_back(0.0); texture.push_back(1.0);
  texture.push_back(1.0); texture.push_back(1.0);

}



void create_drawing_list()
{

  glLineWidth(2.0);

  if(glIsList(drawing_number))
    glDeleteLists(drawing_number,1);
  drawing_number=glGenLists(1);
  glNewList(drawing_number,GL_COMPILE);
  
  int N_quads=connectivity.size();

  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D,texture_number);
  glTexEnvi (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

  glEnableClientState(GL_NORMAL_ARRAY);	
  glEnableClientState(GL_VERTEX_ARRAY);
  glEnableClientState(GL_COLOR_ARRAY);
  glEnableClientState(GL_TEXTURE_COORD_ARRAY);
  
  glNormalPointer(GL_DOUBLE,0,&normal[0]);
  glVertexPointer(3,GL_DOUBLE,0,&vertex[0]);
  glColorPointer(3,GL_DOUBLE,0,&color[0]);
  glTexCoordPointer(2,GL_DOUBLE,0,&texture[0]);
  
  glDrawElements(GL_QUADS,N_quads,GL_UNSIGNED_INT,&connectivity[0]);
  
  glDisableClientState(GL_NORMAL_ARRAY);
  glDisableClientState(GL_COLOR_ARRAY);
  glDisableClientState(GL_VERTEX_ARRAY);
  glDisableClientState(GL_TEXTURE_COORD_ARRAY);
  glDisable(GL_TEXTURE_2D);
  
  glEndList();
}

// load a repeated texture
void load_texture_file(const char* filename)
{
  Texture tex;
  tex.load_picture(filename);

  glPixelStorei (GL_UNPACK_ALIGNMENT, 1);
  glGenTextures (1, &texture_number);
  glBindTexture (GL_TEXTURE_2D, texture_number);

  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);//GL_REPEAT
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

  if(tex.get_depth()==4)
    gluBuild2DMipmaps (GL_TEXTURE_2D, tex.get_depth(), tex.get_Nx(), tex.get_Ny(), GL_RGBA,GL_UNSIGNED_BYTE, tex.get_data());
  if(tex.get_depth()==3)
    gluBuild2DMipmaps (GL_TEXTURE_2D, tex.get_depth(), tex.get_Nx(), tex.get_Ny(), GL_RGB,GL_UNSIGNED_BYTE, tex.get_data());

}


void scale()
{
  int N_vertex=vertex.size()/3;
  double minimum[3]={99,99,99},maximum[3]={-1,-1,-1};
  for(int k=0;k<N_vertex;k++)
    for(int k_dim=0;k_dim<3;k_dim++)
      {
	if(vertex[3*k+k_dim]>maximum[k_dim])
	  maximum[k_dim]=vertex[3*k+k_dim];
	if(vertex[3*k+k_dim]<minimum[k_dim])
	  minimum[k_dim]=vertex[3*k+k_dim];
      }
  double maxmax=fmax(fmax(maximum[0],maximum[1]),maximum[2]);
  double minmin=fmax(fmin(minimum[0],minimum[1]),minimum[2]);
  
  for(int k=0;k<N_vertex;k++)
    for(int k_dim=0;k_dim<3;k_dim++)
      vertex[3*k+k_dim]=(vertex[3*k+k_dim]-0.5*(maximum[k_dim]+minimum[k_dim]))/(maxmax-minmin);
}


void build_normal()
{

  
  int N_quad=connectivity.size()/4;
  int N_vertex=vertex.size()/3;
  face_ring.resize(N_vertex);
  
  //fill face ring
  for(int k=0;k<N_quad;k++)
    for(int k_quad=0;k_quad<4;k_quad++)
      face_ring[connectivity[4*k+k_quad]].push_back(k);
  
  //normal per face
  std::vector <double> face_normal;
  face_normal.resize(3*N_quad);
  double *X0=NULL,*X1=NULL,*X2=NULL;
  double u[3],v[3];
  double _normal[3],n;
  for(int k=0;k<N_quad;k++)
    {
      X0=&vertex[3*connectivity[4*k+0]];
      X1=&vertex[3*connectivity[4*k+1]];
      X2=&vertex[3*connectivity[4*k+2]];
      
      for(int k_dim=0;k_dim<3;u[k_dim]=X1[k_dim]-X0[k_dim],v[k_dim]=X2[k_dim]-X0[k_dim],k_dim++){};
      _normal[0]=u[1]*v[2]-u[2]*v[1];
      _normal[1]=u[2]*v[0]-u[0]*v[2];
      _normal[2]=u[0]*v[1]-u[1]*v[0];
      n = powf(_normal[0]*_normal[0]+_normal[1]*_normal[1]+_normal[2]*_normal[2],0.5);
      if(n>0.0001)
	for(int k_dim=0;k_dim<3;k_dim++)
	  _normal[k_dim]/=n;
      for(int k_dim=0;k_dim<3;k_dim++)
	face_normal[3*k+k_dim]=_normal[k_dim];
    }
  int N_ring=0;
  normal.resize(3*N_vertex);
  for(int k=0;k<N_quad;k++)
    {
      //init to 0
      for(int k_dim=0;k_dim<3;k_dim++)
	normal[3*k+k_dim]=0.0;
      
      //sum up over the faces
      N_ring=face_ring[k].size();
      for(int k_ring=0;k_ring<N_ring;k_ring++)
	for(int k_dim=0;k_dim<3;k_dim++)
	  normal[3*k+k_dim]+=face_normal[3*face_ring[k][k_ring]+k_dim];
      
      //normalize
      n = powf(normal[3*k+0]*normal[3*k+0]+normal[3*k+1]*normal[3*k+1]+normal[3*k+2]*normal[3*k+2],0.5);
      if(n>0.0001)
	for(int k_dim=0;k_dim<3;k_dim++)
	  normal[3*k+k_dim]/=n;
    }
  
}

void affichage()
{

  /* effacement de l'image avec la couleur de fond */
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  
  
  //Projection
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity ();
  gluPerspective (45, 1.0, 0.1,100.0);
  
  
  //Modelview
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glTranslatef(0.0,0.0,zoom);
  glRotatef(-angley,1.0,0.0,0.0);
  glRotatef(-anglex,0.0,1.0,0.0);
  
  
  glCallList(drawing_number);
  
  /*
    glBegin(GL_TRIANGLES);
    
    int N=connectivity.size()/3;
    for(int i=0;i<N;i++)
    {
    for(int k=0;k<3;k++)
    {
    glColor3f(
    color[3*connectivity[3*i+k]+0],
    color[3*connectivity[3*i+k]+1],
    color[3*connectivity[3*i+k]+2]
    );
    
    
    glNormal3f(
    normal[3*connectivity[3*i+k]+0],
    normal[3*connectivity[3*i+k]+1],
    normal[3*connectivity[3*i+k]+2]
    );
    glVertex3f(
    vertex[3*connectivity[3*i+k]+0],
    vertex[3*connectivity[3*i+k]+1],
    vertex[3*connectivity[3*i+k]+2]
    );
    }
    }
    
    glEnd();
  */
  
  /* On echange les buffers */
  glutSwapBuffers();
}

void clavier(unsigned char touche,int x,int y)
{
  x=0;y=0;
  switch (touche)
    {
    case 'p': /* affichage du carre plein */
      glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
      glutPostRedisplay();
      break;
    case 'f': /* affichage en mode fil de fer */
      glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
      glutPostRedisplay();
      break;
    case 's' : /* Affichage en mode sommets seuls */
      glPolygonMode(GL_FRONT_AND_BACK,GL_POINT);
      glutPostRedisplay();
      break;
    case 'd':
      glEnable(GL_DEPTH_TEST);
      glutPostRedisplay();
      break;
    case 'D':
      glDisable(GL_DEPTH_TEST);
      glutPostRedisplay();
      break;
    case 'q' : /*la touche 'q' permet de quitter le programme */
      exit(0);
    }
}

void reshape(int x,int y)
{
	if (x<y)
		glViewport(0,(y-x)/2,x,x);
	else 
		glViewport((x-y)/2,0,y,y);
}

void mouse(int button, int state,int x,int y)
{
	/* si on appuie sur le bouton gauche */
	if(state == GLUT_DOWN) 
	{
		xold = x; /* on sauvegarde la position de la souris */
		yold=y;
	}
	if(button == GLUT_LEFT_BUTTON && state == GLUT_DOWN)
		presse=1;
	if(button == GLUT_LEFT_BUTTON && state==GLUT_UP)
		presse=0;

	if(button == GLUT_RIGHT_BUTTON && state == GLUT_DOWN)
		presse_d=1;
	if(button == GLUT_RIGHT_BUTTON && state == GLUT_UP)
		presse_d=0;

}

void mousemotion(int x,int y)
{
	if (presse) /* si le bouton gauche est presse */
	{
		/* on modifie les angles de rotation de l'objet
		   en fonction de la position actuelle de la souris et de la derniere
		   position sauvegardee */
		anglex=anglex+(x-xold); 
		angley=angley+(y-yold);
		glutPostRedisplay(); /* on demande un rafraichissement de l'affichage */
	}
	if(presse_d)
	{
		zoom = zoom + (y-yold)/50.0;
		glutPostRedisplay(); /* on demande un rafraichissement de l'affichage */
	}


	xold=x; /* sauvegarde des valeurs courante de le position de la souris */
	yold=y;
}






